/*
 *  misc.c
 *
 *  $Id: misc.c,v 1.1 1999/09/30 14:50:28 source Exp $
 *
 *  Miscellaneous functions
 *
 *  The iODBC driver manager.
 *  
 *  Copyright (C) 1995 by Ke Jin <kejin@empress.com> 
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <iodbcinst.h>
#include "inifile.h"
#include "misc.h"

#ifdef _MAC
#  include	<getfpn.h>
#endif

WORD wSystemDSN = USERDSN_ONLY;
WORD configMode = ODBC_BOTH_DSN;

static int
upper_strneq (
    char *s1,
    char *s2,
    int n)
{
  int i;
  char c1, c2;

  for (i = 1; i < n; i++)
	{
		c1 = s1[i];
		c2 = s2[i];

		if (c1 >= 'a' && c1 <= 'z')
		{
			c1 += ('A' - 'a');
		}
		else if (c1 == '\n')
		{
			c1 = '\0';
		}

		if (c2 >= 'a' && c2 <= 'z')
		{
			c2 += ('A' - 'a');
		}
		else if (c2 == '\n')
		{
			c2 = '\0';
		}

		if ((c1 - c2) || !c1 || !c2)
		{
			break;
		}
	}

  return (int) !(c1 - c2);
}

static char *			/* return new position in input str */
readtoken (
    char *istr,			/* old position in input buf */
    char *obuf)			/* token string ( if "\0", then finished ) */
{
  char *start = obuf;

  /* Skip leading white space */
  while (*istr == ' ' || *istr == '\t')
    istr++;

  for (; *istr && *istr != '\n'; istr++)
	{
		char c, nx;

		c = *(istr);
		nx = *(istr + 1);

		if (c == ';')
		{
			for (; *istr && *istr != '\n'; istr++);
			break;
		}
		*obuf = c;
		obuf++;

		if (nx == ';' || nx == '=' || c == '=')
		{
			istr++;
			break;
		}
	}
  *obuf = '\0';

  /* Trim end of token */
  for (; obuf > start && (*(obuf - 1) == ' ' || *(obuf - 1) == '\t');)
    *--obuf = '\0';

  return istr;
}

#if	!defined(WINDOWS) && !defined(WIN32) && !defined(OS2) && !defined(macintosh)
# include <pwd.h>
# define UNIX_PWD
#endif

char *
_iodbcadm_getinifile (char *buf, int size, int bIsInst, int doCreate)
{
#ifdef _MAC
  OSErr result;
  long fldrDid;
  short fldrRef;
#endif
  int i, j;
  char *ptr;

	#ifdef _MAC
		#ifdef __POWERPC__
			j = STRLEN(bIsInst ? ":ODBC Installer Preferences PPC" : ":ODBC Preferences PPC") + 1;
		#else
			j = STRLEN(bIsInst ? ":ODBC Installer Preferences" : ":ODBC Preferences") + 1;
		#endif
	#else
	  j = STRLEN (bIsInst ?  "/odbcinst.ini" : "/odbc.ini") + 1;
	#endif

  if (size < j)
		return NULL;

#if	!defined(UNIX_PWD)
	#ifdef _MAC
		result = FindFolder (kOnSystemDisk, kPreferencesFolderType, kDontCreateFolder, &fldrRef, &fldrDid);
		if(result!=noErr) return NULL;
		ptr = get_full_pathname(fldrDid, fldrRef);

		i = (ptr) ? STRLEN(ptr) : 0;
		if( i == 0 || i > size - j )
		{
			if(ptr) free(ptr);
			return NULL;
		}

		#ifdef __POWERPC__
			STRCPY(buf, ptr); STRCAT(buf, bIsInst ? ":ODBC Installer Preferences PPC" : ":ODBC Preferences PPC");
		#else
			STRCPY(buf, ptr); STRCAT(buf, bIsInst ? ":ODBC Installer Preferences" : ":ODBC Preferences");
		#endif
		free(ptr);
		
		return buf;
	#else
	  /*
  	 *  On Windows, there is only one place to look
	   */
	  i = GetWindowsDirectory ((LPSTR) buf, size);

	  if (i == 0 || i > size - j)
			return NULL;
	
	  sprintf (buf + i, bIsInst ? "/odbcinst.ini" : "/odbc.ini");

	  return buf;
	#endif
#else

  if (wSystemDSN==USERDSN_ONLY)
	{
		/*
		 *  1. Check $ODBCINI environment variable
		 */
		if ((ptr = getenv (bIsInst ? "ODBCINSTINI" : "ODBCINI")) != NULL)
		{
			STRNCPY (buf, ptr, size);

			if (doCreate || access (buf, R_OK) == 0)
				return buf;
		}

#ifdef VMS
		/*
		 *  2a. VMS calls this HOME
		 */
		STRNCPY (buf, bIsInst ? "SYS$LOGIN:ODBCINST.INI" : "SYS$LOGIN:ODBC.INI", size);

		if (doCreate || access (buf, R_OK) == 0)
			return buf;
#else
		/*
     *  2b. Check either $HOME/.odbc.ini or ~/.odbc.ini
     */
		if ((ptr = getenv ("HOME")) == NULL)
		{
			ptr = (char *) getpwuid (getuid ());

			if (ptr != NULL)
				ptr = ((struct passwd *) ptr)->pw_dir;
		}

    if (ptr != NULL)
		{
			sprintf (buf, bIsInst ? "%s/.odbcinst.ini" : "%s/.odbc.ini", ptr);

			if (doCreate || access (buf, R_OK) == 0)
				return buf;
		}

#endif /* VMS */
	}

  /*
   *  3. Try SYS_ODBC_INI as the last resort
   */
  if (wSystemDSN==SYSTEMDSN_ONLY || bIsInst)
	{
		/*
		 *  1. Check $SYSODBCINI environment variable
		 */
		if ((ptr = getenv (bIsInst ? "SYSODBCINSTINI" : "SYSODBCINI")) != NULL)
		{
			STRNCPY (buf, ptr, size);

			if (doCreate || access (buf, R_OK) == 0)
				return buf;
		}

#ifdef _BE
		STRNCPY (buf, bIsInst ? "/boot/beos/etc/odbcinst.ini" : "/boot/beos/etc/odbc.ini", size);
#else
		STRNCPY (buf, bIsInst ? SYS_ODBCINST_INI : SYS_ODBC_INI, size);
#endif

		if (doCreate || access (buf, R_OK) == 0)
			return buf;
	}

  /*
   *  No ini file found or accessible
   */
  return NULL;
#endif /* UNIX_PWD */
}

/* 
 *  read odbc init file to resolve the value of specified
 *  key from named or defaulted dsn section 
 */
char *
_iodbcdm_getkeyvalbydsn (
    char *dsn,
    int dsnlen,
    char *keywd,
    char *value,
    int size)
{
  char buf[1024];
  char dsntk[SQL_MAX_DSN_LENGTH + 3] = {'[', '\0'};
  char token[1024];		/* large enough */
  FILE *file;
  char pathbuf[1024];
  char *path;

#define DSN_NOMATCH	0
#define DSN_NAMED	1
#define DSN_DEFAULT	2

  int dsnid = DSN_NOMATCH;
  int defaultdsn = DSN_NOMATCH;

  if (dsn == NULL || *dsn == 0)
	{
		dsn = "default";
		dsnlen = STRLEN (dsn);
	}

  if (dsnlen == SQL_NTS)
		dsnlen = STRLEN (dsn);

  if (dsnlen <= 0 || keywd == NULL || buf == 0 || size <= 0)
		return NULL;

  if (dsnlen > sizeof (dsntk) - 2)
		return NULL;

  value[0] = '\0';

  STRNCAT (dsntk, dsn, dsnlen);
  STRCAT (dsntk, "]");

  dsnlen = dsnlen + 2;

  path = _iodbcadm_getinifile (pathbuf, sizeof (pathbuf), FALSE, FALSE);

  if (path == NULL)
		return NULL;

  file = (FILE *) fopen (path, "r");

  if (file == NULL)
		return NULL;

  for (;;)
	{
		char *str;

		str = fgets (buf, sizeof (buf), file);

		if (str == NULL) break;

		if (*str == '[')
		{
			if (upper_strneq (str, "[default]", STRLEN ("[default]")))
	    {
	      /* we only read first dsn default dsn
	       * section (as well as named dsn).
	       */
	      if (defaultdsn == DSN_NOMATCH)
				{
					dsnid = DSN_DEFAULT;
					defaultdsn = DSN_DEFAULT;
				}
	      else dsnid = DSN_NOMATCH;

	      continue;
	    }
			else if (upper_strneq (str, dsntk, dsnlen))
	    {
	      dsnid = DSN_NAMED;
	    }
			else
	    {
	      dsnid = DSN_NOMATCH;
	    }

			continue;
		}
		else if (dsnid == DSN_NOMATCH)
		{
			continue;
		}

		str = readtoken (str, token);

		if (upper_strneq (keywd, token, STRLEN (keywd)))
		{
			str = readtoken (str, token);

			if (!STREQ (token, "="))
	    /* something other than = */
	    {
	      continue;
	    }

			str = readtoken (str, token);

			if (STRLEN (token) > size - 1)
	      break;

			STRNCPY (value, token, size);
			/* copy the value(i.e. next token) to buf */

			if (dsnid != DSN_DEFAULT)
	      break;
		}
	}

  fclose (file);

  return (*value) ? value : NULL;
}


char *
_iodbcdm_getkeyvalinstr (
    char *cnstr,
    int cnlen,
    char *keywd,
    char *value,
    int size)
{
  char token[1024] = {'\0'};
  int flag = 0;

  if (cnstr == NULL || value == NULL
      || keywd == NULL || size < 1)
		return NULL;

  if (cnlen == SQL_NTS) cnlen = STRLEN (cnstr);

  if (cnlen <= 0) return NULL;

  for (;;)
	{
		cnstr = readtoken (cnstr, token);

		if (*token == '\0') break;

		if (STREQ (token, ";"))
		{
			flag = 0;
			continue;
		}

		switch (flag)
		{
			case 0:
				if (upper_strneq (token, keywd, strlen (keywd))) flag = 1;
				break;

			case 1:
				if (STREQ (token, "=")) flag = 2;
				break;

			case 2:
				if (size < strlen (token) + 1) return NULL;
				STRNCPY (value, token, size);
				return value;

			default:
				break;
		}
	}

  return NULL;
}

const char *
_iodbcdm_check_for_string(const char *szList, const char *szString, int bContains)
{
  const char *currP = szList, *ret;
  
  while (*currP)
	{
		if (bContains)
		{
			if (strstr(currP, szString)) return currP;
		}
		else
		{
			if (!strcmp(currP, szString)) return currP;
		}
	}
  return NULL;
}

char *
_iodbcdm_remove_quotes(const char *szString)
{
  char *szWork, *szPtr;
 
  while (*szString == '\'' || *szString == '\"')
    szString += 1;
  
  if (!*szString) return NULL;
  szWork = strdup(szString);
  szPtr = strchr(szWork, '\'');
  if (szPtr) *szPtr = 0;
  szPtr = strchr(szWork, '\"');
  if (szPtr) *szPtr = 0;
 
  return szWork; 
}
